<?php

/**
 * @file
 * Rules hook implementations for Signup Rules module.
 */

/**
 * Implements hook_rules_event_info().
 */
function signup_rules_rules_event_info() {
  $events = array();
  $events['signup_rules_event_insert'] = array(
    'label' => t('User signed up for content'),
    'module' => 'Signup',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User who signed up for content')),
      'node' => array('type' => 'node', 'label' => t('Content to which the user signed up')),
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'help' => 'This event is triggered when a user first signs up for content.',
  );
  $events['signup_rules_event_update'] = array(
    'label' => t('Signup was changed'),
    'module' => 'Signup',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User who is signed up for content')),
      'node' => array('type' => 'node', 'label' => t('Content to which the user is signed up')),
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'help' => 'This event is triggered when the user has updated a signup.',
  );
  $events['signup_rules_event_cancel'] = array(
    'label' => t('Signup was cancelled'),
    'module' => 'Signup',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'help' => 'This event is triggered when an individual user\'s signup has been cancelled.',
  );
  $events['signup_rules_event_open'] = array(
    'label' => t('Signups were opened for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content to which signups were opened')),
    ),
    'help' => 'This event is triggered when the status of signups for content is changed to open.',
  );
  $events['signup_rules_event_close'] = array(
  'label' => t('Signups were closed for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content to which signups were closed')),
    ),
    'help' => 'This event is triggered when the status of signups for content is changed to closed.',
  );
  return $events;
}

/**
 * Implements hook_rules_condition_info().
 *
 * Some of these conditions could have opposites implemented, but that
 * is not necessary since Rules was nice enough to provide an inverted
 * value option.
 * @TODO: possible condition, user signed up from registration form?
 */
function signup_rules_condition_info() {
  $conditions = array();
  $conditions['signup_rules_condition_signup_total'] = array(
    'label' => t('Content signup total is equal to'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Signup content')),
    ),
    'help' => 'This condition determines whether the signup total of the content is less than, equal to, or greater than a given numeric value.',
  );
  $conditions['signup_rules_condition_user_is_signed_up'] = array(
    'label' => t('User is signed up for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
      'user' => array('type' => 'user', 'label' => t('User')),
    ),
    'help' => 'This condition determines whether a user is signed up for a piece of content.',
  );
  $conditions['signup_rules_condition_signups_open'] = array(
    'label' => t('Signups are open for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'This condition determines whether signups are open for the content.',
  );
  $conditions['signup_rules_condition_signup_attended'] = array(
    'label' => t('Signup is marked attended'),
    'module' => 'Signup',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'help' => 'This condition determines whether a signup has been marked \'attended\'.',
  );
  return $conditions;
}

/**
 * Form for signup total condition.
 *
 * @param $settings
 *   An array of settings for edited forms.
 * @param &$form
 *   The existing form array. This argument is passed by reference so
 *   we just alter it and that's all.
 */
function signup_rules_condition_signup_total_form($settings, &$form) {
  $form['settings']['signup_rules_condition'] = array(
    '#type' => 'select',
    '#title' => t('Signup total is'),
    '#options' => array(
      'greater' => t('greater than'),
      'equal' => t('equal to'),
      'less' => t('less than'),
    ),
    '#default_value' => $settings['signup_rules_condition'],
    '#required' => TRUE,
  );
  $form['settings']['signup_rules_total'] = array(
    '#type' => 'textfield',
    '#title' => t('value'),
    '#default_value' => $settings['signup_rules_total'],
    '#required' => TRUE,
  );
}

/**
 * Validation callback for signup total condition form.
 */
function signup_rules_condition_signup_total_validate($form, $form_state) {
  if (!is_numeric($form_state['values']['settings']['signup_rules_total'])) {
    form_set_error('signup_rules_total', t('You must enter a numeric value.'));
  }
}

/**
 * Condition: determines whether a signup total for a node has a specific value.
 *
 * @param $node
 *   The node being checked.
 * @param $settings
 *   An array of settings from the condition form. This array has values
 *   from our custom elements to use to determine operators and values
 *   against which we check the number of signups for the node.
 */
function signup_rules_condition_signup_total($node, $settings) {
  $total = db_result(db_query("SELECT COUNT(*) FROM {signup_log} WHERE nid = %d", $node->nid));
  switch ($settings['signup_rules_condition']) {
    case 'greater':
      if ($settings['signup_rules_total'] > $total) {
        return TRUE;
      }
      return FALSE;
    case 'equal':
      if ($settings['signup_rules_total'] == $total) {
        return TRUE;
      }
      return FALSE;
    case 'less':
      if ($settings['signup_rules_total'] < $total) {
        return TRUE;
      }
      return FALSE;
  }
}

/**
 * Condition: checks if user is signed up for a node.
 */
function signup_rules_condition_user_is_signed_up($node, $user) {
  $result = db_query("SELECT uid FROM {signup_log} WHERE uid = %d AND nid = %d", $user->uid, $node->nid);
  return db_fetch_object($result) ? TRUE : FALSE;
}

/**
 * Condition: checks if signups are open for a node.
 *
 * We check the signup status of the node by checking the $node
 * object. This object is populated with signup data in hook_nodeapi(),
 * so we should have no problem getting the information here.
 */
function signup_rules_condition_signups_open($node) {
  return $node->signup_status == 1 ? TRUE : FALSE;
}

/**
 * Condition: checks if a signup has been marked attended.
 */
function signup_rules_condition_signup_attended($signup) {
  return $signup->attended == 1 ? TRUE : FALSE;
}

/**
 * Implements hook_rules_action_info().
 */
function signup_rules_action_info() {
  $actions = array();
  $actions['signup_rules_action_cancel_node_signups'] = array(
    'label' => t('Cancel all signups for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'This action cancels all signups for a given piece of content.',
  );
  $actions['signup_rules_action_cancel_user_signups'] = array(
    'label' => t('Cancel all signups for user'),
    'module' => 'Signup',
    'arguments' => array(
      'user' => array('type' => 'user', 'label' => t('User')),
    ),
    'help' => 'This action cancels all signups for a given user.',
  );
  $actions['signup_rules_action_open_signups'] = array(
    'label' => t('Open signups for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'This action opens up signups for a piece of content.',
  );
  $actions['signup_rules_action_close_signups'] = array(
    'label' => t('Close signups for content'),
    'module' => 'Signup',
    'arguments' => array(
      'node' => array('type' => 'node', 'label' => t('Content')),
    ),
    'help' => 'This action opens up signups for a piece of content.',
  );
  $actions['signup_rules_action_mark_attended'] = array(
    'label' => t('Mark signup as attended'),
    'module' => 'Signup',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'help' => 'This action marks a signup as attended.',
  );
  $actions['signup_rules_action_mark_not_attended'] = array(
    'label' => t('Mark signup as not attended'),
    'module' => 'Signup',
    'arguments' => array(
      'signup' => array('type' => 'signup', 'label' => t('Signup data')),
    ),
    'help' => 'This action marks a signup as not attended.',
  );
  return $actions;
}

/**
 * Action: cancels all signups for a node.
 */
function signup_rules_action_cancel_node_signups($node, $settings) {
  $signups = signup_get_signups($node->nid);
  foreach ($signups as $signup) {
    signup_cancel_signup($signup, FALSE);
  }
  return;
}

/**
 * Action: cancel all signups for a user.
 */
function signup_rules_action_cancel_user_signups($user) {
  $signups = signup_rules_get_user_signups($user->uid);
  foreach ($signups as $signup) {
    signup_cancel_signup($signup, FALSE);
  }
}

/**
 * Action: closes signups on a node.
 *
 * The conditional checks to see if signups are even enabled for
 * the node type. If they aren't then there is no attempt to close
 * them as they should be closed by Signup already.
 */
function signup_rules_action_close_signups($node) {
  if (variable_get('signup_node_default_state_'. $node->type, 'disabled') != 'disabled') {
    signup_close_signup($node->nid);
  }
}

/**
 * Action: opens signups on a node.
 *
 * The conditional checks to ensure that signups are enabled for
 * the node before programatically opening signups.
 */
function signup_rules_action_open_signups($node) {
  if (variable_get('signup_node_default_state_'. $node->type, 'disabled') != 'disabled') {
    signup_open_signup($node->nid);
  }
}

/**
 * Action: marks a signup as attended.
 */
function signup_rules_action_mark_attended($signup) {
  signup_mark_attended_action($signup);
}

/**
 * Action: marks a signup as not attended.
 */
function signup_rules_action_mark_not_unattended($signup) {
  signup_mark_not_attended_action($signup);
}

/**
 * Implements hook_rules_date_type_info().
 *
 * Tokens are made available by Signup module, so we make use of their
 * Token type. Also, our data type class makes use of save and load
 * functions, so this data type is saveable.
 */
function signup_rules_data_type_info() {
  return array(
    'signup' => array(
      'label' => t('Signup data'),
      'class' => 'rules_data_type_signup',
      'module' => 'Signup',
      'token type' => 'signup',
      'savable' => TRUE,
      'identifiable' => TRUE,
    ),
  );
}

/**
 * Defines the signup data type.
 */
class rules_data_type_signup extends rules_data_type {

  function save() {
    $signup = &$this->get();
    signup_save_signup($signup);
    return TRUE;
  }

  function load($sid) {
    return signup_load_signup($sid);
  }

  function get_identifier() {
    $signup = $this->get();
    return $signup->sid;
  }
}
